@@ -1013,13 +1013,14 @@ srv_export_innodb_status(void)
 
 	if (buf_dblwr.is_initialised()) {
 		buf_dblwr.lock();
-		dblwr = buf_dblwr.submitted();
-		export_vars.innodb_dblwr_pages_written = buf_dblwr.written();
+		dblwr = buf_dblwr.written();
+		export_vars.innodb_dblwr_pages_written = dblwr;
 		export_vars.innodb_dblwr_writes = buf_dblwr.batches();
 		buf_dblwr.unlock();
 	}
 
-	export_vars.innodb_data_written = srv_stats.data_written + dblwr;
+	export_vars.innodb_data_written = srv_stats.data_written
+		+ (dblwr << srv_page_size_shift);
 
 	export_vars.innodb_buffer_pool_read_requests
 		= buf_pool.stat.n_page_gets;
